#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <vector>
#include <cassert>
#include <pthread.h>
#include <sys/time.h>
#include <time.h>
using std::cin;
typedef long long int64;
typedef int key_t;
typedef long long int value_t;

const int max_threads_num = 32;
const int max_queries = 100 * 1000 + 1;

struct Query {
	enum QueryType { CMD_PUT = 1, 
		CMD_REMOVE = 2, 
		CMD_GET = 3,
		CMD_END = 4 };

	int cmd;
	int timestamp;
	key_t key;
	value_t value;

	Query() {}
	~Query() { }

	void read() {
		cin >> cmd;
		if(cmd != CMD_END)
			cin >> key;
		if(cmd == CMD_PUT || cmd == CMD_GET)
			cin >> value;
		cin >> timestamp;
	}
};

#include "task.cpp"
using namespace std;

int threads_num;
vector<Query> qs[max_threads_num];
pthread_barrier_t exec_query_barrier;
Task *task;

void readQueries(const char *filename = NULL) {
	if(filename != NULL) {
		freopen(filename, "r", stdin);
	}
	ios_base::sync_with_stdio(false);
	cin >> threads_num;
	assert(threads_num <= max_threads_num);
	for(int i = 0; i < threads_num; i++) {
		int queries_count;
		cin >> queries_count;
		assert(queries_count <= max_queries);
		for(int j = 0; j < queries_count; j++) {
			qs[i].push_back(Query());
			qs[i].back().read();
		}
	}
}

void* execQueries(void *thread_id_p) {
	const long thread_id = reinterpret_cast<long>(thread_id_p);

	pthread_barrier_wait(&exec_query_barrier);
	bool fail = false;
	for(vector<Query>::iterator it = qs[thread_id].begin(); it != qs[thread_id].end(); ++it) {
		if(it->cmd == Query::CMD_PUT)
			task->put(it->key, it->value, it->timestamp);
		else if(it->cmd == Query::CMD_REMOVE)
			task->remove(it->key, it->timestamp);
		else if(it->cmd == Query::CMD_END)
			task->end(it->timestamp);
		else {
			value_t user_result = task->get(it->key, it->timestamp);
			if(user_result != it->value) {
				fail = true;
			}
		}
	}

	return (void*)(fail);
}

void run() {
	task = new Task((const int)(threads_num));
	if(task == NULL) {
		perror("epic fail");
		exit(1);
	}

	if(pthread_barrier_init(&exec_query_barrier, NULL, threads_num + 1)) {
		perror("can't create barrier");
		exit(1);
	}
	pthread_t user_threads[threads_num];
	for(int i = 0; i < threads_num; i++) {
		if(pthread_create(user_threads + i, NULL, execQueries, reinterpret_cast<void*>(long(i)))) {
			perror("can't create thread");
			exit(1);
		}
	}

	timeval tv_str, tv_end;
	pthread_barrier_wait(&exec_query_barrier);
	gettimeofday(&tv_str, NULL);
	int64 real_time = -(int64(tv_str.tv_sec) * 1000000 + tv_str.tv_usec);

	bool fail = false;
	for(int i = 0; i < threads_num; i++) {
		long res;
		pthread_join(user_threads[i], (void**)(&res));
		if(res) {
			fail = true;
		}
	}
	gettimeofday(&tv_end, NULL);
	real_time += int64(tv_end.tv_sec) * 1000000 + tv_end.tv_usec;

	if(fail)
		fprintf(stderr, "0\n");
	else 
		fprintf(stderr, "1 %d\n", int((real_time / 1000) + .5));
}

int main(int, char**) {
	readQueries("input.txt");
	run();

	return 0;
}
