import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

public final class Executable {
	private static final String INP_FILE = "input.txt";
	private static final int NANO_TO_MILLI = 1000000;
	
	private static final int NULL_KEY = -1;
	private static final long NULL_VALUE = Long.MIN_VALUE;
	
	private static class Command {
		private static final int CMD_PUT = 1;
		private static final int CMD_REMOVE = 2;
		private static final int CMD_GET = 3;
		private static final int CMD_END = 4;
		
		public final int cmd;
		public final int key;
		public final long val;
		public final int ts;
		
		public Command(final int cmd, final int key, final long val, final int ts) {
			this.cmd = cmd;
			this.key = key;
			this.val = val;
			this.ts = ts;
		}
	}
	
	private Command[][] threadsQuerries;
	private Thread[] pool;
	private Task userSolution;
	private boolean wrong = false;
	private int threadsNum = -1;
	
	private void readThreadsQuerries() throws IOException { 
		final BufferedReader inp = new BufferedReader(new FileReader(INP_FILE));
		
		StringTokenizer st = new StringTokenizer(inp.readLine());
		threadsNum = Integer.parseInt(st.nextToken());
		threadsQuerries = new Command[threadsNum][];
		for (int i = 0; i < threadsNum; ++i) {
			st = new StringTokenizer(inp.readLine());
			final int querriesNum = Integer.parseInt(st.nextToken());
			threadsQuerries[i] = new Command[querriesNum];
			for (int j = 0; j < querriesNum; ++j) {
				st = new StringTokenizer(inp.readLine());
				final int cmd = Integer.parseInt(st.nextToken());
				int key = NULL_KEY;
				long val = NULL_VALUE;
				if (Command.CMD_END != cmd) {
					key = Integer.parseInt(st.nextToken());
					if (Command.CMD_REMOVE != cmd) {
						val = Long.parseLong(st.nextToken());
					}
				}
				final int timestamp = Integer.parseInt(st.nextToken());
				threadsQuerries[i][j] = new Command(cmd, key, val, timestamp);
			}
		}
		
		inp.close();
	}
	
	private void prepareThreads() {
		pool = new Thread[threadsNum];
		for (int i = 0; i < threadsNum; ++i) {
			final int curThread = i;
			pool[i] = new Thread(new Runnable() {
				@Override
				public void run() {
					try {
						final Command[] querries = threadsQuerries[curThread]; 
						final int querriesNum = querries.length;
						for (int j = 0; j < querriesNum; ++j) {
							final Command q = querries[j];
							if (Command.CMD_PUT == q.cmd) {
								userSolution.put(q.key, q.val, q.ts);
							} else if (Command.CMD_REMOVE == q.cmd) {
								userSolution.remove(q.key, q.ts);
							} else if (Command.CMD_END == q.cmd) {
								userSolution.end(q.ts);
							} else if (Command.CMD_GET == q.cmd) {
								if (q.val != userSolution.get(q.key, q.ts)) {
									wrongSolution();
								}
							} else {
								throw new IllegalStateException();
							}
						}
					} catch (Exception ex) {
						wrongSolution();
					}
				}
			});
		}
	}
	
	private synchronized void wrongSolution() {
		wrong = true;
	}
	
	private void invokeUserSolution() {
		for (int i = 0; i < threadsNum; ++i) {
			pool[i].start();
		}
		
		for (int i = 0; i < threadsNum; ++i) {
			try {
				pool[i].join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void invoke() {
		try {
			readThreadsQuerries();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		prepareThreads();
		userSolution = new Task(threadsNum);
		invokeUserSolution();
		
		prepareThreads();
		userSolution = null;
		System.gc();
		userSolution = new Task(threadsNum);
		final long startTime = System.nanoTime();
		invokeUserSolution();
		final long estimatedTime = System.nanoTime() - startTime;
		if (wrong) {
			System.err.println(0);
		} else {
			System.err.println(1 + " " + (int)((double)estimatedTime / NANO_TO_MILLI + 0.5));
		}
		System.err.flush();
	}
	
	public static void main(String[] args) {
		new Executable().invoke();
	}
}
