{$mode objfpc}
uses task, cthreads, cmem, SysUtils, DateUtils;

const
	THREAD_MAX = 32;
	QUERRIES_MAX = 100 * 1000;

	CMD_PUT = 1;
	CMD_REMOVE = 2;
	CMD_GET = 3;
	CMD_END = 4;

type
	querry = record
		cmd : longint;
		key : longint;
		val : int64;
		timestamp : longint;
	end;

var
	thq : array[0..THREAD_MAX, 0..QUERRIES_MAX] of querry;
	thN : longint;
	qN : array[0..THREAD_MAX] of longint;
	thid : array[0..THREAD_MAX] of tthreadid;
	utask : ttask;
	cs : TRTLCriticalSection;
	wrong : boolean;

procedure inp;
var i, j, curqN : longint;
begin
	assign(input, 'input.txt'); reset(input);
	read(thN); dec(thN);
	for i := 0 to thN do begin
		read(qN[i]); dec(qN[i]);
		curqN := qN[i];
		for j := 0 to curqN do begin
			read(thq[i][j].cmd);
			if CMD_END <> thq[i][j].cmd then begin
				read(thq[i][j].key);
				if CMD_REMOVE <> thq[i][j].cmd then begin
					read(thq[i][j].val);
				end;
			end;
			read(thq[i][j].timestamp);
		end;
	end;
	close(input);
end;

procedure wrongSolution;
begin
	EnterCriticalSection(cs);
	wrong := true;
	LeaveCriticalSection(cs);
end;

function worker(p : pointer) : ptrint;
var i, n, curqn : longint;
begin
	n := longint(p);
	curqn := qN[n];
	for i := 0 to curqn do begin
		if CMD_PUT = thq[n][i].cmd then begin
			utask.put(thq[n][i].key, thq[n][i].val, thq[n][i].timestamp);
		end else if CMD_REMOVE = thq[n][i].cmd then begin
			utask.remove(thq[n][i].key, thq[n][i].timestamp);
		end else if CMD_END = thq[n][i].cmd then begin
			utask.finish(thq[n][i].timestamp);
		end else if CMD_GET = thq[n][i].cmd then begin
			if thq[n][i].val <> utask.get(thq[n][i].key, thq[n][i].timestamp) then begin
				wrongSolution;
			end;
		end;
	end;

	worker := 0;
end;

procedure invoke;
var i : longint;
	bt, et : TDateTime;
begin
	utask := ttask.init(thN + 1);
	bt := TimeOf(Now);

	for i := 0 to thN do begin
		thid[i] := beginthread(@worker, pointer(i));
	end;
	for i := 0 to thN do begin
		WaitForThreadTerminate(thid[i], 0);
	end;
	
	et := TimeOf(Now);
	if wrong then begin
		writeln(stderr, 0);
	end else begin
		writeln(stderr, 1, ' ', MilliSecondSpan(bt, et):0:0);
	end;
	flush(stderr);
end;

begin
	InitCriticalSection(cs);
	wrong := false;
	inp;
	invoke;
	DoneCriticalSection(cs);
end.
